@@ -33,8 +33,10 @@ import java.util.concurrent.Future;
 import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.concurrent.locks.ReentrantLock;
 
 import org.apache.hc.client5.http.DnsResolver;
+import org.apache.hc.client5.http.EndpointInfo;
 import org.apache.hc.client5.http.HttpRoute;
 import org.apache.hc.client5.http.SchemePortResolver;
 import org.apache.hc.client5.http.config.ConnectionConfig;
@@ -47,9 +49,8 @@ import org.apache.hc.client5.http.io.HttpClientConnectionManager;
 import org.apache.hc.client5.http.io.HttpClientConnectionOperator;
 import org.apache.hc.client5.http.io.LeaseRequest;
 import org.apache.hc.client5.http.io.ManagedHttpClientConnection;
-import org.apache.hc.client5.http.socket.ConnectionSocketFactory;
-import org.apache.hc.client5.http.socket.PlainConnectionSocketFactory;
-import org.apache.hc.client5.http.ssl.SSLConnectionSocketFactory;
+import org.apache.hc.client5.http.ssl.DefaultClientTlsStrategy;
+import org.apache.hc.client5.http.ssl.TlsSocketStrategy;
 import org.apache.hc.core5.annotation.Contract;
 import org.apache.hc.core5.annotation.Internal;
 import org.apache.hc.core5.annotation.ThreadingBehavior;
@@ -119,41 +120,65 @@ public class PoolingHttpClientConnectionManager
     private volatile Resolver<HttpHost, TlsConfig> tlsConfigResolver;
 
     public PoolingHttpClientConnectionManager() {
-        this(RegistryBuilder.<ConnectionSocketFactory>create()
-                .register(URIScheme.HTTP.id, PlainConnectionSocketFactory.getSocketFactory())
-                .register(URIScheme.HTTPS.id, SSLConnectionSocketFactory.getSocketFactory())
-                .build());
+        this(new DefaultHttpClientConnectionOperator(null, null,
+                RegistryBuilder.<TlsSocketStrategy>create()
+                    .register(URIScheme.HTTPS.id, DefaultClientTlsStrategy.createDefault())
+                    .build()),
+                PoolConcurrencyPolicy.STRICT,
+                PoolReusePolicy.LIFO,
+                TimeValue.NEG_ONE_MILLISECOND,
+                null);
     }
 
+    /**
+     * @deprecated Use {@link PoolingHttpClientConnectionManagerBuilder}
+     */
+    @Deprecated
     public PoolingHttpClientConnectionManager(
-            final Registry<ConnectionSocketFactory> socketFactoryRegistry) {
+            final Registry<org.apache.hc.client5.http.socket.ConnectionSocketFactory> socketFactoryRegistry) {
         this(socketFactoryRegistry, null);
     }
 
+    /**
+     * @deprecated Use {@link PoolingHttpClientConnectionManagerBuilder}
+     */
+    @Deprecated
     public PoolingHttpClientConnectionManager(
-            final Registry<ConnectionSocketFactory> socketFactoryRegistry,
+            final Registry<org.apache.hc.client5.http.socket.ConnectionSocketFactory> socketFactoryRegistry,
             final HttpConnectionFactory<ManagedHttpClientConnection> connFactory) {
         this(socketFactoryRegistry, PoolConcurrencyPolicy.STRICT, TimeValue.NEG_ONE_MILLISECOND, connFactory);
     }
 
+    /**
+     * @deprecated Use {@link PoolingHttpClientConnectionManagerBuilder}
+     */
+    @Deprecated
     public PoolingHttpClientConnectionManager(
-            final Registry<ConnectionSocketFactory> socketFactoryRegistry,
+            final Registry<org.apache.hc.client5.http.socket.ConnectionSocketFactory> socketFactoryRegistry,
             final PoolConcurrencyPolicy poolConcurrencyPolicy,
             final TimeValue timeToLive,
             final HttpConnectionFactory<ManagedHttpClientConnection> connFactory) {
         this(socketFactoryRegistry, poolConcurrencyPolicy, PoolReusePolicy.LIFO, timeToLive, connFactory);
     }
 
+    /**
+     * @deprecated Use {@link PoolingHttpClientConnectionManagerBuilder}
+     */
+    @Deprecated
     public PoolingHttpClientConnectionManager(
-            final Registry<ConnectionSocketFactory> socketFactoryRegistry,
+            final Registry<org.apache.hc.client5.http.socket.ConnectionSocketFactory> socketFactoryRegistry,
             final PoolConcurrencyPolicy poolConcurrencyPolicy,
             final PoolReusePolicy poolReusePolicy,
             final TimeValue timeToLive) {
         this(socketFactoryRegistry, poolConcurrencyPolicy, poolReusePolicy, timeToLive, null);
     }
 
+    /**
+     * @deprecated Use {@link PoolingHttpClientConnectionManagerBuilder}
+     */
+    @Deprecated
     public PoolingHttpClientConnectionManager(
-            final Registry<ConnectionSocketFactory> socketFactoryRegistry,
+            final Registry<org.apache.hc.client5.http.socket.ConnectionSocketFactory> socketFactoryRegistry,
             final PoolConcurrencyPolicy poolConcurrencyPolicy,
             final PoolReusePolicy poolReusePolicy,
             final TimeValue timeToLive,
@@ -161,8 +186,12 @@ public class PoolingHttpClientConnectionManager
         this(socketFactoryRegistry, poolConcurrencyPolicy, poolReusePolicy, timeToLive, null, null, connFactory);
     }
 
+    /**
+     * @deprecated Use {@link PoolingHttpClientConnectionManagerBuilder}
+     */
+    @Deprecated
     public PoolingHttpClientConnectionManager(
-            final Registry<ConnectionSocketFactory> socketFactoryRegistry,
+            final Registry<org.apache.hc.client5.http.socket.ConnectionSocketFactory> socketFactoryRegistry,
             final PoolConcurrencyPolicy poolConcurrencyPolicy,
             final PoolReusePolicy poolReusePolicy,
             final TimeValue timeToLive,
@@ -177,7 +206,7 @@ public class PoolingHttpClientConnectionManager
     }
 
     @Internal
-    protected PoolingHttpClientConnectionManager(
+    public PoolingHttpClientConnectionManager(
             final HttpClientConnectionOperator httpClientConnectionOperator,
             final PoolConcurrencyPolicy poolConcurrencyPolicy,
             final PoolReusePolicy poolReusePolicy,
@@ -296,75 +325,82 @@ public class PoolingHttpClientConnectionManager
         }
         final Future<PoolEntry<HttpRoute, ManagedHttpClientConnection>> leaseFuture = this.pool.lease(route, state, requestTimeout, null);
         return new LeaseRequest() {
-
+            // Using a ReentrantLock specific to each LeaseRequest instance to maintain the original
+            // synchronization semantics. This ensures that each LeaseRequest has its own unique lock.
+            private final ReentrantLock lock = new ReentrantLock();
             private volatile ConnectionEndpoint endpoint;
 
             @Override
-            public synchronized ConnectionEndpoint get(
+            public ConnectionEndpoint get(
                     final Timeout timeout) throws InterruptedException, ExecutionException, TimeoutException {
-                Args.notNull(timeout, "Operation timeout");
-                if (this.endpoint != null) {
-                    return this.endpoint;
-                }
-                final PoolEntry<HttpRoute, ManagedHttpClientConnection> poolEntry;
+                lock.lock();
                 try {
-                    poolEntry = leaseFuture.get(timeout.getDuration(), timeout.getTimeUnit());
-                } catch (final TimeoutException ex) {
-                    leaseFuture.cancel(true);
-                    throw ex;
-                }
-                if (LOG.isDebugEnabled()) {
-                    LOG.debug("{} endpoint leased {}", id, ConnPoolSupport.formatStats(route, state, pool));
-                }
-                final ConnectionConfig connectionConfig = resolveConnectionConfig(route);
-                try {
-                    if (poolEntry.hasConnection()) {
-                        final TimeValue timeToLive = connectionConfig.getTimeToLive();
-                        if (TimeValue.isNonNegative(timeToLive)) {
-                            if (timeToLive.getDuration() == 0
-                                    || Deadline.calculate(poolEntry.getCreated(), timeToLive).isExpired()) {
-                                poolEntry.discardConnection(CloseMode.GRACEFUL);
+                    Args.notNull(timeout, "Operation timeout");
+                    if (this.endpoint != null) {
+                        return this.endpoint;
+                    }
+                    final PoolEntry<HttpRoute, ManagedHttpClientConnection> poolEntry;
+                    try {
+                        poolEntry = leaseFuture.get(timeout.getDuration(), timeout.getTimeUnit());
+                    } catch (final TimeoutException ex) {
+                        leaseFuture.cancel(true);
+                        throw ex;
+                    }
+                    if (LOG.isDebugEnabled()) {
+                        LOG.debug("{} endpoint leased {}", id, ConnPoolSupport.formatStats(route, state, pool));
+                    }
+                    final ConnectionConfig connectionConfig = resolveConnectionConfig(route);
+                    try {
+                        if (poolEntry.hasConnection()) {
+                            final TimeValue timeToLive = connectionConfig.getTimeToLive();
+                            if (TimeValue.isNonNegative(timeToLive)) {
+                                if (timeToLive.getDuration() == 0
+                                        || Deadline.calculate(poolEntry.getCreated(), timeToLive).isExpired()) {
+                                    poolEntry.discardConnection(CloseMode.GRACEFUL);
+                                }
                             }
                         }
-                    }
-                    if (poolEntry.hasConnection()) {
-                        final TimeValue timeValue = resolveValidateAfterInactivity(connectionConfig);
-                        if (TimeValue.isNonNegative(timeValue)) {
-                            if (timeValue.getDuration() == 0
-                                    || Deadline.calculate(poolEntry.getUpdated(), timeValue).isExpired()) {
-                                final ManagedHttpClientConnection conn = poolEntry.getConnection();
-                                boolean stale;
-                                try {
-                                    stale = conn.isStale();
-                                } catch (final IOException ignore) {
-                                    stale = true;
-                                }
-                                if (stale) {
-                                    if (LOG.isDebugEnabled()) {
-                                        LOG.debug("{} connection {} is stale", id, ConnPoolSupport.getId(conn));
+                        if (poolEntry.hasConnection()) {
+                            final TimeValue timeValue = resolveValidateAfterInactivity(connectionConfig);
+                            if (TimeValue.isNonNegative(timeValue)) {
+                                if (timeValue.getDuration() == 0
+                                        || Deadline.calculate(poolEntry.getUpdated(), timeValue).isExpired()) {
+                                    final ManagedHttpClientConnection conn = poolEntry.getConnection();
+                                    boolean stale;
+                                    try {
+                                        stale = conn.isStale();
+                                    } catch (final IOException ignore) {
+                                        stale = true;
+                                    }
+                                    if (stale) {
+                                        if (LOG.isDebugEnabled()) {
+                                            LOG.debug("{} connection {} is stale", id, ConnPoolSupport.getId(conn));
+                                        }
+                                        poolEntry.discardConnection(CloseMode.IMMEDIATE);
                                     }
-                                    poolEntry.discardConnection(CloseMode.IMMEDIATE);
                                 }
                             }
                         }
+                        final ManagedHttpClientConnection conn = poolEntry.getConnection();
+                        if (conn != null) {
+                            conn.activate();
+                        } else {
+                            poolEntry.assignConnection(connFactory.createConnection(null));
+                        }
+                        this.endpoint = new InternalConnectionEndpoint(poolEntry);
+                        if (LOG.isDebugEnabled()) {
+                            LOG.debug("{} acquired {}", id, ConnPoolSupport.getId(endpoint));
+                        }
+                        return this.endpoint;
+                    } catch (final Exception ex) {
+                        if (LOG.isDebugEnabled()) {
+                            LOG.debug("{} endpoint lease failed", id);
+                        }
+                        pool.release(poolEntry, false);
+                        throw new ExecutionException(ex.getMessage(), ex);
                     }
-                    final ManagedHttpClientConnection conn = poolEntry.getConnection();
-                    if (conn != null) {
-                        conn.activate();
-                    } else {
-                        poolEntry.assignConnection(connFactory.createConnection(null));
-                    }
-                    this.endpoint = new InternalConnectionEndpoint(poolEntry);
-                    if (LOG.isDebugEnabled()) {
-                        LOG.debug("{} acquired {}", id, ConnPoolSupport.getId(endpoint));
-                    }
-                    return this.endpoint;
-                } catch (final Exception ex) {
-                    if (LOG.isDebugEnabled()) {
-                        LOG.debug("{} endpoint lease failed", id);
-                    }
-                    pool.release(poolEntry, false);
-                    throw new ExecutionException(ex.getMessage(), ex);
+                } finally {
+                    lock.unlock();
                 }
             }
 
@@ -387,6 +423,11 @@ public class PoolingHttpClientConnectionManager
         if (LOG.isDebugEnabled()) {
             LOG.debug("{} releasing endpoint", ConnPoolSupport.getId(endpoint));
         }
+
+        if (this.isClosed()) {
+            return;
+        }
+
         final ManagedHttpClientConnection conn = entry.getConnection();
         if (conn != null && keepAlive == null) {
             conn.close(CloseMode.GRACEFUL);
@@ -438,22 +479,22 @@ public class PoolingHttpClientConnectionManager
             poolEntry.assignConnection(connFactory.createConnection(null));
         }
         final HttpRoute route = poolEntry.getRoute();
-        final HttpHost host = route.getProxyHost() != null ? route.getProxyHost() : route.getTargetHost();
+        final HttpHost firstHop = route.getProxyHost() != null ? route.getProxyHost() : route.getTargetHost();
         final SocketConfig socketConfig = resolveSocketConfig(route);
         final ConnectionConfig connectionConfig = resolveConnectionConfig(route);
-        final TlsConfig tlsConfig = resolveTlsConfig(host);
         final Timeout connectTimeout = timeout != null ? Timeout.of(timeout.getDuration(), timeout.getTimeUnit()) : connectionConfig.getConnectTimeout();
         if (LOG.isDebugEnabled()) {
-            LOG.debug("{} connecting endpoint to {} ({})", ConnPoolSupport.getId(endpoint), host, connectTimeout);
+            LOG.debug("{} connecting endpoint to {} ({})", ConnPoolSupport.getId(endpoint), firstHop, connectTimeout);
         }
         final ManagedHttpClientConnection conn = poolEntry.getConnection();
         this.connectionOperator.connect(
                 conn,
-                host,
+                firstHop,
+                route.getTargetName(),
                 route.getLocalSocketAddress(),
                 connectTimeout,
                 socketConfig,
-                tlsConfig,
+                route.isTunnelled() ? null : resolveTlsConfig(route.getTargetHost()),
                 context);
         if (LOG.isDebugEnabled()) {
             LOG.debug("{} connected {}", ConnPoolSupport.getId(endpoint), ConnPoolSupport.getId(conn));
@@ -470,9 +511,14 @@ public class PoolingHttpClientConnectionManager
         final InternalConnectionEndpoint internalEndpoint = cast(endpoint);
         final PoolEntry<HttpRoute, ManagedHttpClientConnection> poolEntry = internalEndpoint.getValidatedPoolEntry();
         final HttpRoute route = poolEntry.getRoute();
-        final HttpHost host = route.getProxyHost() != null ? route.getProxyHost() : route.getTargetHost();
-        final TlsConfig tlsConfig = resolveTlsConfig(host);
-        this.connectionOperator.upgrade(poolEntry.getConnection(), route.getTargetHost(), tlsConfig, context);
+        final HttpHost target = route.getTargetHost();
+        final TlsConfig tlsConfig = resolveTlsConfig(target);
+        this.connectionOperator.upgrade(
+                poolEntry.getConnection(),
+                target,
+                route.getTargetName(),
+                tlsConfig,
+                context);
     }
 
     @Override
@@ -481,11 +527,17 @@ public class PoolingHttpClientConnectionManager
         if (LOG.isDebugEnabled()) {
             LOG.debug("Closing connections idle longer than {}", idleTime);
         }
+        if (isClosed()) {
+            return;
+        }
         this.pool.closeIdle(idleTime);
     }
 
     @Override
     public void closeExpired() {
+        if (isClosed()) {
+            return;
+        }
         LOG.debug("Closing expired connections");
         this.pool.closeExpired();
     }
@@ -539,7 +591,7 @@ public class PoolingHttpClientConnectionManager
      * Sets the same {@link SocketConfig} for all routes
      */
     public void setDefaultSocketConfig(final SocketConfig config) {
-        this.socketConfigResolver = (route) -> config;
+        this.socketConfigResolver = route -> config;
     }
 
     /**
@@ -557,7 +609,7 @@ public class PoolingHttpClientConnectionManager
      * @since 5.2
      */
     public void setDefaultConnectionConfig(final ConnectionConfig config) {
-        this.connectionConfigResolver = (route) -> config;
+        this.connectionConfigResolver = route -> config;
     }
 
     /**
@@ -575,7 +627,7 @@ public class PoolingHttpClientConnectionManager
      * @since 5.2
      */
     public void setDefaultTlsConfig(final TlsConfig config) {
-        this.tlsConfigResolver = (host) -> config;
+        this.tlsConfigResolver = host -> config;
     }
 
     /**
@@ -702,6 +754,10 @@ public class PoolingHttpClientConnectionManager
             getValidatedPoolEntry().getConnection().setSocketTimeout(timeout);
         }
 
+        /**
+         * @deprecated Use {@link #execute(String, ClassicHttpRequest, RequestExecutor, HttpContext)}
+         */
+        @Deprecated
         @Override
         public ClassicHttpResponse execute(
                 final String exchangeId,
@@ -717,5 +773,52 @@ public class PoolingHttpClientConnectionManager
             return requestExecutor.execute(request, connection, context);
         }
 
+        /**
+         * @since 5.4
+         */
+        @Override
+        public ClassicHttpResponse execute(
+                final String exchangeId,
+                final ClassicHttpRequest request,
+                final RequestExecutor requestExecutor,
+                final HttpContext context) throws IOException, HttpException {
+            Args.notNull(request, "HTTP request");
+            Args.notNull(requestExecutor, "Request executor");
+            final ManagedHttpClientConnection connection = getValidatedPoolEntry().getConnection();
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("{} executing exchange {} over {}", id, exchangeId, ConnPoolSupport.getId(connection));
+            }
+            return requestExecutor.execute(request, connection, context);
+        }
+
+        /**
+         * @since 5.4
+         */
+        @Override
+        public EndpointInfo getInfo() {
+            final PoolEntry<HttpRoute, ManagedHttpClientConnection> poolEntry = poolEntryRef.get();
+            if (poolEntry != null) {
+                final ManagedHttpClientConnection connection = poolEntry.getConnection();
+                if (connection != null && connection.isOpen()) {
+                    return new EndpointInfo(connection.getProtocolVersion(), connection.getSSLSession());
+                }
+            }
+            return null;
+        }
+
     }
+
+    /**
+     * Method that can be called to determine whether the connection manager has been shut down and
+     * is closed or not.
+     *
+     * @return {@code true} if the connection manager has been shut down and is closed, otherwise
+     * return {@code false}.
+     * @since 5.4
+     */
+    public boolean isClosed() {
+        return this.closed.get();
+    }
+
+
 }
